/*
 * Copyright (c) 2003, Takahide Matsutsuka.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Takahide Matsutsuka."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * hex2cas conversion functionality.
 */

#include <stdio.h>

int prefix_length = 30;
#define DEFAULT_P6FILENAME "Defalt";
char prefix[] = { 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 
		  0xd3, 0xd3, 0x47, 0x6f, 0x00, 0x00, 0x00, 0x00, 
		  0x0d, 0x84, 0x0a, 0x00, 0xa5, 0x26, 0x48, 0x38,
		  0x34, 0x31, 0x32, 0x00, 0x00, 0x00 };

//  char prefix[] = { 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 0xd3, 
//  		  0xd3, 0xd3, 0x47, 0x6f, 0x00, 0x00, 0x00, 0x00, 
//  		  0x0d, 0x84, 0x0a, 0x00, 0xa5, 0x26, 0x48, 0x38,
//  		  0x34, 0x30, 0x46, 0x00, 0x00, 0x00 };

// write GS_INIT on 8411, 8412
// write _main on 8414, 8415
#define ADDRESS_STARTUP 0x0100

// startup is actually 12 byte long, but it causes error due to rst8,
// so we stop it at 9 byteo
#define STARTUP_LENGTH 9
//  #define STARTUP_LENGTH 12
#define ADDRESS_CODESEG 0x841b
int suffix_length = 12;
char suffix[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		  0x00, 0x00, 0x00, 0x00 };

unsigned char memory[65536];
int maxAddress;

int aton(unsigned char ch) {
  int n;
  if (ch >= '0' && ch <= '9') {
    return ch - '0';
  }
  if (ch >= 'A' && ch <= 'F') {
    return ch - 'A' + 10;
  }
  if (ch >= 'a' && ch <= 'f') {
    return ch - 'a' + 10;
  }

  return 0;
}

// -1 if EOF
int getByte(FILE *in) {
  int ch1, ch2;
  ch1 = getc(in);
  if (ch1 == EOF) {
    return -1;
  }
  ch2 = getc(in);
  if (ch2 == EOF) {
    return -1;
  }
  return 16 * aton(ch1) + aton(ch2);
}


int hex2mem(const char *inFilename) {
  FILE *in;
  maxAddress = 0;

  memset(memory, 0, 65536);

  in = fopen(inFilename, "rb");
  if (in == NULL) {
    printf("cannot open input file\n");
    return 1;
  }

  while(1) {
    int ch;
    int length;
    int address;
    int tmp;

    while ((ch = getc(in)) != ':') {
      if (ch == EOF) {
	break;
      }
    }
    if (ch == EOF) {
      break;
    }

    // get length of a chunk
    length = getByte(in);
    if (length <= 0) {
      break;
    }

    // make an address
    tmp = getByte(in);
    if (tmp < 0) {
      break;
    }
    address = tmp * 256;
    tmp = getByte(in);
    if (tmp < 0) {
      break;
    }
    address += tmp;    
    if (maxAddress < (address + length)) {
      maxAddress = address + length;
    }

    // ignore record type
    if (getByte(in) < 0) {
      break;
    }

    while (length > 0) {
      memory[address] = getByte(in);
      address++;
      length--;
    }
  }

  fclose(in);
  return 0;
}

int mem2cas(const char *outFilename, const char p6Filename[6]) {
  FILE *out;
  int ch;
  int i;

  if (p6Filename[0] == 0) {
    p6Filename = DEFAULT_P6FILENAME;
  }

  // set p6 file name
  for (i = 0; i < 6; i++) {
    prefix[i + 0x0a] = p6Filename[i];
  }

  // open a file
  out = fopen(outFilename, "wb");
  if (out == NULL) {
    printf("cannot open output file\n");
    return 1;
  }

  // write prefix
  for (i = 0; i < prefix_length; i++) {
    putc(prefix[i], out);
  }

  // startup routine looks like:
  // 840F 31FFFF    LD      SP,$FFFF
  // 8412 CDD984    CALL    $84D9
  // 8415 CDB984    CALL    $84B9
  // 8418 C31F84    JP      $841F

  // 841F 3E02      LD      A,$02
  // 8421 CF        RST     $08
  // 8422 C9        RET     
  // somehow sdcc doesn't emit other than 0xffff for sp,
  // and PC-6001 causes an error for rst 0x8,
  // we use some workaround instead:
  //   1/ start from 0x8412 (written in prefix)
  //   2/ write c9 on 0x8418 (for above example)

  // write startup routine
  for (i = ADDRESS_STARTUP; i < ADDRESS_STARTUP + STARTUP_LENGTH; i++) {
    putc(memory[i], out);
  }

  // return at the end of startup code instead of rst8
  putc(0xc9, out);
  putc(0x00, out);
  putc(0x00, out);

  // write codeseg
  for (i = ADDRESS_CODESEG; i < maxAddress; i++) {
    putc(memory[i], out);
  }

  // write suffix
  for (i = 0; i < suffix_length; i++) {
    putc(suffix[i], out);
  }

  fclose(out);

  return 0;
}
