/*
 * Copyright (c) 2003, Takahide Matsutsuka.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Takahide Matsutsuka."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 * 4. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.  
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * main file for hex2cas.
 */

#include <unistd.h>
#include <string.h>
#include <stdio.h>

char* inFilename;
char* outFilename;
char p6Filename[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
char *changeExt(const char *pathName, const char* ext);
int analyzeOption(int argc, char **argv);
int bin2cas(const char *inFilename, const char *outFilename, const char p6Filename[6]);


int main(int argc, char **argv) {
  inFilename = NULL;
  outFilename = NULL;

  printf("%s : Convert Intel hex file to PC-6001 cas file version 1.0.0\n", argv[0]);
  printf("Copyright (C) 2003 Takahide Matsutsuka <markn@markn.org>\n");
  printf("\n");

  if (analyzeOption(argc, argv)) {
    printf("unknown option: %s\n", optopt);
    return 1;
  }

  inFilename = argv[optind];
  optind++;
  if (optind < argc) {
    if (analyzeOption(argc, argv)) {
      printf("unknown option: %s\n", optopt);
      return 1;
    }
  }
  if (optind < argc) {
    printf("unknown argument: ");
    for(; optind < argc; optind++){
      printf("%s ", argv[optind]);
    }
    printf("\n");
    return 1;
  }

  if (!inFilename) {
    printf("no input specified\n");
    return 1;
  }

//    printf("in=%s\n", inFilename);
  if (outFilename == NULL) {
    outFilename = changeExt(inFilename, "cas");
  }

  printf("Generating file: %s\n", outFilename);
  if (hex2mem(inFilename) == 0) {
    return mem2cas(outFilename, p6Filename);
  }
  return -1;
}

char *changeExt(const char *pathName, const char* ext) {
  char *p;
  char *tail;
  char *changed;
  int len;
  int extlen = strlen(ext);

  for (tail = (char*) pathName; *tail != 0; tail++);
  for (p = tail; p > pathName; p--) {
    if (*p == '.') {
      len = p - pathName;
      changed = (char*) malloc(len + extlen + 2);

      strncpy(changed, inFilename, len + 1);
      strcpy(changed + len + 1, ext);
      return changed;
    }
  }

  len = strlen(pathName);
  changed = (char*) malloc(len + extlen + 2);
  strncpy(changed, inFilename, len);
  *(changed + len) = '.';
  strcpy(changed + len + 1, ext);
  return changed;
}


int analyzeOption(int argc, char **argv) {
  int c;
  int i;
  opterr = 0;
  while((c = getopt(argc, argv, "o:n:")) != EOF){
    switch(c){
    case 'o':
      outFilename = optarg;
      break;
    case 'n':
      for (i = 0; i < 6; i++) {
	if (optarg[i] == 0) {
	  break;
	}
	p6Filename[i] = optarg[i];
      }
      break;
    case '?':
      return -1;
    }
  }

  return 0;
}

